/////////////////////////////////////////////////////////////////////////////
//  File:       FormulaCtrl.h
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//  parts of Code by Chris Maunder
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
// It's free
//
/////////////////////////////////////////////////////////////////////////////


#if !defined(_FORMULACTRL_H__INCLUDED_)
#define _FORMULACTRL_H__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// scrollbar selection
#define FCSB_NONE				0L
#define FCSB_HORZ				1L
#define FCSB_VERT				2L
#define FCSB_BOTH				3L

#define IDC_INPLACE_CONTROL		8

// Handy functions
#define IsSHIFTpressed() ( (GetKeyState(VK_SHIFT) & (1 << (sizeof(SHORT)*8-1))) != 0   )
#define IsCTRLpressed()  ( (GetKeyState(VK_CONTROL) & (1 << (sizeof(SHORT)*8-1))) != 0 )

/////////////////////////////////////////////////////////////////////////////
// CFormulaCtrl window

#include "BinTree.h"
#include "FormulaInPlaceEdit.h"
#include "FormulaDropTarget.h"

#define FORMULACTRL_CLASSNAME _T("MFCFormulaCtrl")  // Window class name

#define IDC_INPLACE_CONTROL	8

class CFormulaCtrl : public CWnd
{
    DECLARE_DYNCREATE(CFormulaCtrl)
// Construction
public:
	CFormulaCtrl();
	virtual ~CFormulaCtrl();

	// setter and getter for the binary tree
	void SetBinTree(CBinTree* pBinTree);
	CBinTree* GetBinTree();
	UINT CF_BINTREE;

// Attributes
protected:

	// some scrolling attributes
    int	m_nBarState;
    int m_nVScrollMax;
	int m_nHScrollMax;
	int m_nRowsPerWheelNotch;
	CRect m_VirtualRect;

	// pointer to in place edit control
	CFormulaInPlaceEdit* m_pEditWnd;

	// pointer to the binary tree data structure
	CBinTree* m_pBinTree;

	// indicates read only mode
	BOOL m_bReadOnly;
	BOOL m_bPreparedToDrag;
	BOOL m_bPreparedToEdit;
// Operations
public:
	BOOL Create(CWnd* pParentWnd, const RECT& rect, UINT nID, DWORD dwStyle = WS_VISIBLE);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CFormulaCtrl)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL
	CNode* m_pFocusNode;

	// OLE Drop target for the control
    CFormulaDropTarget m_DropTarget;
	
public:
// Implementation
	CRect GetVirtualRect();
	void EnableScrollBars(int nBar, BOOL bEnable /*=TRUE*/);
	void ResetScrollBars();
	int  GetScrollPos32(int nBar, BOOL bGetTrackPos = FALSE);
    BOOL SetScrollPos32(int nBar, int nPos, BOOL bRedraw = TRUE);
    BOOL IsVisibleVScroll() { return ( (m_nBarState & FCSB_VERT) > 0); } 
    BOOL IsVisibleHScroll() { return ( (m_nBarState & FCSB_HORZ) > 0); }
	void SetReadOnly(BOOL bReadOnly);
	
///////////////////////////////////////////////////////////////////////////////////
// Clipboard, drag and drop, and cut n' paste operations
///////////////////////////////////////////////////////////////////////////////////
public:
    virtual void CutSubTree();
    virtual COleDataSource* CopySubTree();
    virtual BOOL PasteSubTree(CNode* pNode, COleDataObject* pDataObject);

    virtual void OnBeginDrag();
    virtual DROPEFFECT OnDragEnter(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);
    virtual DROPEFFECT OnDragOver(COleDataObject* pDataObject, DWORD dwKeyState, CPoint point);
    virtual void OnDragLeave();
    virtual BOOL OnDrop(COleDataObject* pDataObject, DROPEFFECT dropEffect, CPoint point);

    virtual void OnEditCut();
    virtual void OnEditCopy();
    virtual void OnEditPaste();
    virtual void OnEditSelectAll();

protected:
    BOOL RegisterWindowClass();
	LRESULT SendMessageToParent(int nMessage) const;
  
	// Drawing 
    virtual void  OnDraw(CDC* pDC);
	
	// Transformation of coordinates (supports scrolling)
	void TransformPoint(CPoint& point);
	CRect TransformRect(CRect& rect);

	// Generated message map functions
protected:
	//{{AFX_MSG(CFormulaCtrl)
	afx_msg void OnPaint();
    afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
    afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnSize(UINT nType, int nCx, int nCy);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
    afx_msg void OnEndInPlaceEdit(NMHDR* pNMHDR, LRESULT* pResult);
    afx_msg void OnUpdateEditCopy(CCmdUI* pCmdUI);
    afx_msg void OnUpdateEditCut(CCmdUI* pCmdUI);
    afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
    afx_msg void OnUpdateEditSelectAll(CCmdUI* pCmdUI);
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(_FORMULACTRL_H__INCLUDED_)
