/////////////////////////////////////////////////////////////////////////////
//  File:       FormulaDlg.cpp
//  Version:    1.0.0.0
//  Created:    03-april-2002
//
//  Author:     Thorsten Wack
//  E-mail:     wt@umsicht.fhg.de
//
//	FormulaDlg.cpp : implementation file
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "mathsym.h"
#include "FormulaDlg.h"
#include "FormulaCtrl.h"
#include "nodeentities.h"
#include "formulasettingsdlg.h"
#include "mainfrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFormulaDlg dialog


CFormulaDlg::CFormulaDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CFormulaDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CFormulaDlg)
	m_strOutput = _T("");
	//}}AFX_DATA_INIT
}

CFormulaDlg::~CFormulaDlg()
{
	m_OutpuCtrlFont.DeleteObject();
}

void CFormulaDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFormulaDlg)
	DDX_Control(pDX, IDC_COMBO_LANG_TYPE, m_ComboLangType);
	DDX_Control(pDX, IDC_EDIT_OUTPUT, m_EditOutput);
	DDX_Control(pDX, IDC_BUTTON_REDO,	m_btnRedo);
	DDX_Control(pDX, IDC_BUTTON_UNDO,	m_btnUndo);
	DDX_Text(pDX, IDC_EDIT_OUTPUT, m_strOutput);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_EQUATION,		m_btnEquation);
	DDX_Control(pDX, IDC_BRACE,			m_btnBrace);
	DDX_Control(pDX, IDC_UPPER_GREEK,	m_btnUpperGreek);
	DDX_Control(pDX, IDC_LOWER_GREEK,	m_btnLowerGreek);
	DDX_Control(pDX, IDC_SUM,			m_btnSum);
	DDX_Control(pDX, IDC_PROD,			m_btnProd);
	DDX_Control(pDX, IDC_INTEGRAL,		m_btnIntegral);
	DDX_Control(pDX, IDC_DERIVATIVE,	m_btnDerivative);
	DDX_Control(pDX, IDC_MATRIX,		m_btnMatrix);
	DDX_Control(pDX, IDC_PBYQ,			m_btnPbyq);
	DDX_Control(pDX, IDC_FUNCTION,		m_btnFunction);
	DDX_Control(pDX, IDC_COREDATA,		m_btnCoreData);
	DDX_Control(pDX, IDC_VECTOR,		m_btnVector);
	DDX_Control(pDX, IDC_ARROWS,		m_btnArrows);
    DDX_Control(pDX, IDC_CUSTOM,		m_FormulaCtrl);
}

BEGIN_MESSAGE_MAP(CFormulaDlg, CDialog)
	//{{AFX_MSG_MAP(CFormulaDlg)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_BUTTON_UNDO, OnButtonUndo)
	ON_BN_CLICKED(IDC_BUTTON_REDO, OnButtonRedo)
	ON_BN_CLICKED(IDC_BUTTON_SETTINGS, OnButtonSettings)
	ON_BN_CLICKED(IDC_EXPORT_BMP, OnExportBmp)
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_BN_CLICKED(IDC_EXPORT_SRC, OnExportSrc)
	//}}AFX_MSG_MAP
	ON_MESSAGE(CS_SELECTED, ToolBarCtrlSelected)
    ON_UPDATE_COMMAND_UI(IDC_BUTTON_UNDO, OnUpdateButtonUndo)
    ON_UPDATE_COMMAND_UI(IDC_BUTTON_REDO, OnUpdateButtonRedo)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFormulaDlg message handlers

BOOL CFormulaDlg::OnInitDialog() 
{
	m_btnUndo.AutoLoad(IDC_BUTTON_UNDO, this);
	m_btnRedo.AutoLoad(IDC_BUTTON_REDO, this);
	
	m_FormulaCtrl.SetBinTree(&m_BinTree);
	GetBinTree()->CheckPoint();

	CDialog::OnInitDialog();
	
	m_ComboLangType.SetCurSel(0);

	m_FormulaCtrl.EnableScrollBarCtrl(SB_BOTH, TRUE);
	m_FormulaCtrl.Invalidate(TRUE);

	// Init the buttons( 
	// no. items in one row, 
	// Bitmap containing the items, 
	// witdh of one item , 
	// transparent bg-color) see mathsym.rc for the bitmaps
	m_btnEquation.InitButton	(3, IDB_EQUATION,		21, RGB(255,255,255));
	m_btnBrace.InitButton		(5, IDB_BRACE,			21, RGB(255,255,255));
	m_btnUpperGreek.InitButton	(4, IDB_UPPER_GREEK,	21, RGB(255,255,255));
	m_btnLowerGreek.InitButton	(4, IDB_LOWER_GREEK,	21, RGB(255,255,255));
	m_btnSum.InitButton			(1, IDB_SUM,			21, RGB(255,255,255));
	m_btnProd.InitButton		(1, IDB_PROD,			21, RGB(255,255,255));
	m_btnIntegral.InitButton	(3, IDB_INTEGRAL,		21, RGB(255,255,255));
	m_btnDerivative.InitButton	(2, IDB_DERIVATIVE,		21, RGB(255,255,255));
	m_btnMatrix.InitButton		(3, IDB_MATRIX,			21, RGB(255,255,255));
	m_btnPbyq.InitButton		(3, IDB_PBYQ,			21, RGB(255,255,255));
	m_btnFunction.InitButton	(4, IDB_FUNCTION,		32, RGB(255,255,255));
	m_btnCoreData.InitButton	(1, IDB_COREDATA,		32, RGB(255,255,255));
	m_btnVector.InitButton		(3, IDB_VECTOR,			21, RGB(255,255,255));
	m_btnArrows.InitButton		(3, IDB_ARROWS,			21, RGB(255,255,255));

	// Create the ToolTip control.
	m_tooltip.Create(this);
	m_tooltip.Activate(TRUE);

	// Add the tools
	m_tooltip.AddTool(GetDlgItem(IDC_BUTTON_UNDO), "Undo");
	m_tooltip.AddTool(GetDlgItem(IDC_BUTTON_REDO), "Redo");

	m_tooltip.AddTool(GetDlgItem(IDC_EQUATION),		"Equations");
	m_tooltip.AddTool(GetDlgItem(IDC_BRACE),		"Braces");
	m_tooltip.AddTool(GetDlgItem(IDC_UPPER_GREEK),	"Upper Greek");
	m_tooltip.AddTool(GetDlgItem(IDC_LOWER_GREEK),	"Lower Greek");
	m_tooltip.AddTool(GetDlgItem(IDC_SUM),			"Sums");
	m_tooltip.AddTool(GetDlgItem(IDC_PROD),			"Products");
	m_tooltip.AddTool(GetDlgItem(IDC_INTEGRAL),		"Integrals");
	m_tooltip.AddTool(GetDlgItem(IDC_DERIVATIVE),	"Derivatives");
	m_tooltip.AddTool(GetDlgItem(IDC_MATRIX),		"Matrices");
	m_tooltip.AddTool(GetDlgItem(IDC_PBYQ),			"Operators");
	m_tooltip.AddTool(GetDlgItem(IDC_FUNCTION),		"Functions");
	m_tooltip.AddTool(GetDlgItem(IDC_COREDATA),		"Core Data");
	m_tooltip.AddTool(GetDlgItem(IDC_VECTOR),		"Vectors");
	m_tooltip.AddTool(GetDlgItem(IDC_ARROWS),		"Arrows");

	//	Create a fix-spaced font to use in output edit control.
	CClientDC dc(this);
	int	nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 9) / 72);

	m_OutpuCtrlFont.CreateFont
	(
		nHeight,
		0,
		0,
		0,
		FW_NORMAL,
		0,
		0,
		0,
		DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS,
		CLIP_CHARACTER_PRECIS,
		DEFAULT_QUALITY,
		DEFAULT_PITCH | FF_DONTCARE,
		_T("Courier New")
	);
	m_EditOutput.SetFont(&m_OutpuCtrlFont);
	
	return TRUE;
}

void CFormulaDlg::OnSize(UINT nType, int nCx, int nCy) 
{
	CDialog::OnSize(nType, nCx, nCy);
}

CBinTree* CFormulaDlg::GetBinTree()
{
	// returns the data-structure of the binary tree
	// this is used in the file FormulaDlg_ToolBar which handles
	// the callbacks of the ToolBar
	return (&m_BinTree);
}

void CFormulaDlg::OnButtonUndo() 
{
	// performs the undo operation and redraws the tree
	GetBinTree()->Undo();
	m_FormulaCtrl.Invalidate(TRUE);
}

void CFormulaDlg::OnButtonRedo() 
{
	// performs the redo operation and redraws the tree
	GetBinTree()->Redo();		
	m_FormulaCtrl.Invalidate(TRUE);
}

void CFormulaDlg::OnOK() 
{
	CDialog::OnOK();
}

void CFormulaDlg::OnCancel() 
{
	CDialog::OnCancel();
}

BOOL CFormulaDlg::PreTranslateMessage(MSG* pMsg)
{
	// Let the ToolTip process this message.
	m_tooltip.RelayEvent(pMsg);
	UpdateDialogControls( this, FALSE );

	return CDialog::PreTranslateMessage(pMsg);
}

void CFormulaDlg::OnButtonSettings() 
{
	// Modify the default formatting behaviour of the tree
	CFormulaSettingsDlg FormulaSettingsDlg;
	FormulaSettingsDlg.m_BinTreeFormat = (*GetFormat());
	if(FormulaSettingsDlg.DoModal()==IDOK)
	{
		(*GetFormat())=FormulaSettingsDlg.m_BinTreeFormat;
		GetBinTree()->ReformatNodes();
		m_FormulaCtrl.Invalidate(TRUE);
	}
}

void CFormulaDlg::OnExportBmp() 
{
	//Write the content into a File using the WriteDIB or WriteWMF method
	static char BASED_CODE szFilter[] = 
		"Windows-Bitmap (*.bmp)|*.bmp|Windows-Metafile (*.wmf)|*.wmf|All Files (*.*)|*.*|";
	
	CFileDialog fdlg( FALSE, "bmp", "*.bmp", OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT , szFilter, NULL );
	fdlg.m_ofn.lpstrTitle="Export Bitmap";
			
	if (fdlg.DoModal() == IDOK)
	{
		CString strFile=fdlg.m_ofn.lpstrFile;
		BOOL bRet=FALSE;
		if(fdlg.GetFileExt()=="wmf")
			bRet=GetBinTree()->WriteWMF(strFile);
		if(fdlg.GetFileExt()=="bmp")
			bRet=GetBinTree()->WriteDIB(strFile);
			
		if(bRet==TRUE)
			AfxMessageBox("Export OK!", MB_ICONINFORMATION);
		else
			AfxMessageBox("Export-failure!", MB_ICONSTOP);
	}
}

void CFormulaDlg::OnExportSrc() 
{
	int nLangType=m_ComboLangType.GetCurSel();
	CString strPutput=GetBinTree()->WriteFormula(nLangType);
	m_EditOutput.SetWindowText(strPutput);

}

void CFormulaDlg::OnUpdateButtonRedo(CCmdUI* pCmdUI)
{
	// update the state of the redo button
	m_btnRedo.EnableWindow(GetBinTree()->CanRedo());
}

void CFormulaDlg::OnUpdateButtonUndo(CCmdUI* pCmdUI)
{
	// update the state of the undo button
	m_btnUndo.EnableWindow(GetBinTree()->CanUndo());
}
